#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include "gfx.h"
#include "image.h"
#include "colbits.h"

int display_image(unsigned int *pix, int xsz, int ysz);
void halve_image(unsigned int *dest, unsigned int *src, int xsz, int ysz);

int stdres[][2] = {
	{320, 240},
	{640, 480},
	{800, 600},
	{1024, 768}
};
int num_stdres = sizeof stdres / sizeof stdres[0];


int main(int argc, char **argv)
{
	int i;
	int xsz, ysz;
	void *img;

	for(i=1; i<argc; i++) {
		if(!(img = load_image(argv[i], &xsz, &ysz))) {
			fprintf(stderr, "failed to load image: %s\n", argv[i]);
			continue;
		}
		display_image(img, xsz, ysz);
	}
	return 0;
}


int display_image(unsigned int *pix, int xsz, int ysz)
{
	int xres, yres, i, j;
	unsigned int *fb;

	/* --- dump as well --- */
	FILE *fp;
	if((fp = fopen("dump.ppm", "wb"))) {
		fprintf(fp, "P6\n%d %d\n255\n", xsz, ysz);
		for(i=0; i<xsz*ysz; i++) {
			unsigned int r = (pix[i] & 0xff0000) >> 16;
			unsigned int g = (pix[i] & 0x00ff00) >> 8;
			unsigned int b = pix[i] & 0x0000ff;
			fputc(r, fp);
			fputc(g, fp);
			fputc(b, fp);
		}
		fclose(fp);
	}
	/* -------------------- */

	for(i=0; i<num_stdres; i++) {
		if(xsz <= stdres[i][0] && ysz <= stdres[i][1]) {
			break;
		}
	}
	if(i >= num_stdres) {
		i = num_stdres - 1;
	}
	xres = stdres[i][0];
	yres = stdres[i][1];

	while(xsz > stdres[i][0] || ysz > stdres[i][1]) {
		unsigned int *newpix;
		int nx = xsz >> 1, ny = ysz >> 1;

		if(!(newpix = malloc(nx * ny * sizeof *newpix))) {
			perror("failed to resize image");
			return -1;
		}
		halve_image(newpix, pix, xsz, ysz);

		free(pix);
		pix = newpix;
		xsz = nx;
		ysz = ny;
	}

	printf("displaying image: %dx%d in display mode: %dx%d\n", xsz, ysz, xres, yres);

	if(!(fb = set_video_mode(xres, yres, 32))) {
		fprintf(stderr, "failed to set video mode: %dx%d\n", xres, yres);
		return -1;
	}
	memset(fb, 0, xres * yres * sizeof *fb);
	vsync();

	for(i=0; i<ysz; i++) {
		for(j=0; j<xsz; j++) {
			*fb++ = *pix++;
		}
		fb += xres - xsz;
	}

	getch();
	set_text_mode();
	return 0;
}

void halve_image(unsigned int *dest, unsigned int *src, int xsz, int ysz)
{
	int i, j;
	int nx = xsz >> 1;
	int ny = ysz >> 1;

	for(i=0; i<ny; i++) {
		for(j=0; j<nx; j++) {
			int r = UNP_RED32(*src);
			int g = UNP_GREEN32(*src);
			int b = UNP_BLUE32(*src);

			r += UNP_RED32(*(src + 1));
			g += UNP_GREEN32(*(src + 1));
			b += UNP_BLUE32(*(src + 1));

			r += UNP_RED32(*(src + xsz));
			g += UNP_GREEN32(*(src + xsz));
			b += UNP_BLUE32(*(src + xsz));

			r += UNP_RED32(*(src + xsz + 1));
			g += UNP_GREEN32(*(src + xsz + 1));
			b += UNP_BLUE32(*(src + xsz + 1));

			*dest++ = PACK_COLOR24(r >> 2, g >> 2, b >> 2);
			src += 2;
		}
		src += xsz;
	}
}
