#include <stdio.h>
#include <stdlib.h>
#include "gfx.h"
#include "gl.h"
#include "scene.h"

int init(void);
void cleanup(void);
void disp(void);
int handle_event(SDL_Event *ev);

SDL_Surface *fb;

int xsz = 320;
int ysz = 240;
unsigned int start_time;

struct scene *scn;

int main(int argc, char **argv)
{
	if(init() == -1) {
		cleanup();
		return 1;
	}

	for(;;) {
		SDL_Event ev;

		if(SDL_PollEvent(&ev)) {
			if(handle_event(&ev) == -1) {
				break;
			}
		} else {
			disp();
		}
	}

	cleanup();
	return 0;
}

int init(void)
{
	SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER | SDL_INIT_JOYSTICK);

	if(!(fb = SDL_SetVideoMode(xsz, ysz, 32, SDL_SWSURFACE))) {
		fprintf(stderr, "SDL init failed\n");
		return -1;
	}

	fglCreateContext();
	glViewport(0, 0, xsz, ysz);

	glClearColor(0, 0, 0, 0);
	glClearDepth(1.0);

	glEnable(GL_DEPTH_TEST);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0, (float)xsz / (float)ysz, 5.0, 200.0);

	if(!(scn = load_scene("scene.dump"))) {
		return -1;
	}

	start_time = SDL_GetTicks();
	return 0;
}

void cleanup(void)
{
	fglDestroyContext();
	SDL_Quit();
}

void disp(void)
{
	uint32_t *pixels;
	unsigned int msec = SDL_GetTicks() - start_time;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0, 0, -80);
	glRotatef(25, 1, 0, 0);
	glRotatef((float)msec / 20.0, 0, 1, 0);

	draw_scene(scn);

	pixels = fglGetFrameBuffer();

	SDL_LockSurface(fb);
	memcpy(fb->pixels, pixels, xsz * ysz * 4);
	SDL_UnlockSurface(fb);

	SDL_Flip(fb);
}

int handle_event(SDL_Event *ev)
{
	switch(ev->type) {
	case SDL_KEYDOWN:
		if(ev->key.keysym.sym == 27) {
			return -1;
		}
		break;

#ifdef GP2X
	case SDL_JOYBUTTONDOWN:
		if(ev->jbutton.button == GP2X_VK_FX) {
			return -1;
		}
		break;
#endif

	default:
		break;
	}
	return 0;
}
