#include <stdio.h>
#include <stdlib.h>
#include "scene.h"
#include "gl.h"

struct scene *load_scene(const char *fname)
{
	FILE *fp;
	struct scene *scn;
	int num_obj, i, j;

	if(!(fp = fopen(fname, "rb"))) {
		return 0;
	}
	fread(&num_obj, sizeof num_obj, 1, fp);

	if(!(scn = malloc(sizeof *scn))) {
		fclose(fp);
		return 0;
	}
	scn->num_obj = num_obj;
	if(!(scn->obj = malloc(num_obj * sizeof *scn->obj))) {
		free(scn);
		fclose(fp);
		return 0;
	}

	for(i=0; i<num_obj; i++) {
		int num_verts;
		struct mesh *m = scn->obj + i;

		if(fread(&num_verts, sizeof num_verts, 1, fp) < 1) {
			free_scene(scn);
			fclose(fp);
			return 0;
		}

		m->vert = malloc(num_verts * sizeof *m->vert);
		m->col = malloc(num_verts * sizeof *m->col);

		for(j=0; j<num_verts; j++) {
			int idx = j;
			if(idx % 3 == 1) {
				idx++;
			} else if(idx % 3 == 2) {
				idx--;
			}
			fread(&m->vert[idx].x, sizeof m->vert[idx].x, 1, fp);
			fread(&m->vert[idx].y, sizeof m->vert[idx].y, 1, fp);
			fread(&m->vert[idx].z, sizeof m->vert[idx].z, 1, fp);

			fread(&m->col[idx].x, sizeof m->col[idx].x, 1, fp);
			fread(&m->col[idx].y, sizeof m->col[idx].y, 1, fp);
			fread(&m->col[idx].z, sizeof m->col[idx].z, 1, fp);
		}

		m->num_vert = num_verts;
	}

	fclose(fp);
	return scn;
}

void free_scene(struct scene *scn)
{
	int i;
	for(i=0; scn->num_obj; i++) {
		free(scn->obj[i].vert);
		free(scn->obj[i].col);
	}
	free(scn->obj);
}

void draw_scene(struct scene *scn)
{
	int i, j;

	for(i=0; i<scn->num_obj; i++) {
		glBegin(GL_TRIANGLES);

		for(j=0; j<scn->obj[i].num_vert; j++) {
			vec3_t *vert = scn->obj[i].vert + j;
			vec4_t *col = scn->obj[i].col + j;

			glColor4f(col->x, col->y, col->z, col->w);
			glVertex3f(vert->x, vert->y, vert->z);
		}

		glEnd();
	}
}
