#include <math.h>
#include <memory.h>
#include <conio.h>
#include <dos.h>
#include "video.h"

const unsigned long VideoBufferSize = 64000ul;

VGA::VGA() {
	_asm {
		mov ax,0x13
		int 0x10
	}

	VideoMem = (BYTE*)MK_FP(0xA000, 0);
	BackBuffer = new BYTE[VideoBufferSize];

	memset(VideoMem, 0, VideoBufferSize);
	memset(BackBuffer, 0, VideoBufferSize);
}

VGA::~VGA() {
	_asm {
		mov ax,0x3
		int 0x10
	}

	delete [] BackBuffer;
}

// Clears screen to color
void VGA::Clear(BYTE c, BYTE *where) {
	memset(where, c, 64000);
}

//void VGA::PutPixel(int x, int y, unsigned char c, unsigned char *where) {
//	where[(y<<6)+(y<<8)+x] = c;
//}


void VGA::GetPal(BYTE &R, BYTE &G, BYTE &B, BYTE index) {
	outp(0x03c7, index);
	R=inp(0x3c9);
	G=inp(0x3c9);
	B=inp(0x3c9);
}

void VGA::SetPal(BYTE R, BYTE G, BYTE B, BYTE index) {
	asm {
		mov dx, 0x3c8
		mov al, index
		out dx, al
		mov al, R
		inc dx
		out dx, al
		mov al, G
		out dx, al
		mov al, B
		out dx, al
   }
   /*
	outp(0x3c8, index);
	outp(0x3c9, R);
	outp(0x3c9, G);
	outp(0x3c9, B);
   */
}

void VGA::WaitRetrace() {

	asm mov dx,0x03DA
l1:
	asm {
		in al,dx
		and al,0x08
		jnz l1
	}
l2:
	asm {
		in al,dx
		and al,0x08
		jz l2
	}
}

// Sets all pallete entries to rgb 0,0,0 (black)
void VGA::BlackOut() {
	for(int i=0;i<256;i++) {
		SetPal(0,0,0,i);
	}
}

// Puts the whole Pallette into Pal array
void VGA::GrabPal() {
	for(int i=0;i<256;i++) {
		GetPal(Pal[i][0], Pal[i][1], Pal[i][2], i);
	}
}


void VGA::FadeIn() {
	unsigned char temp[3];
	GrabPal();
	for(int j=0;j<64;j++) {
		WaitRetrace();
		for(int i=0;i<256;i++) {
			GetPal(temp[0], temp[1], temp[2], i);
			if(temp[0] < Pal[i][0] && temp[0] < 63) {temp[0]++; }
			if(temp[1] < Pal[i][1] && temp[1] < 63) {temp[1]++; }
			if(temp[2] < Pal[i][2] && temp[2] < 63) {temp[2]++; }

			SetPal(temp[0], temp[1], temp[2], i);
		}
	}
}

void VGA::FadeOut() {
	unsigned char temp[3];
	for(int j=0;j<64;j++) {
		WaitRetrace();
		for(int i=0;i<256;i++) {
			GetPal(temp[0], temp[1], temp[2], i);
			if(temp[0] > 0) {temp[0]--; }
			if(temp[1] > 0) {temp[1]--; }
			if(temp[2] > 0) {temp[2]--; }

			SetPal(temp[0], temp[1], temp[2], i);
		}
	}
}

void VGA::RestorePallette() {
	for(int i=0;i<256;i++) {
		SetPal(Pal[i][0], Pal[i][1], Pal[i][2], i);
	}
}

void VGA::Circle(int x, int y, const int radius, int c, BYTE *where) {
	int X = 0;
	int Y = radius;
	int sw = 3 - 2*radius;

	do {

		PutPixel((X+x), (Y+y), c, where);
		PutPixel((X+x), -(Y+y), c, where);
		PutPixel(-(X+x), (Y+y), c, where);
		PutPixel(-(X+x), -(Y+y), c, where);
		PutPixel((Y+y), (X+x), c, where);
		PutPixel((Y+y), -(X+x), c, where);
		PutPixel(-(Y+y), (X+x), c, where);
		PutPixel(-(Y+y), -(X+x), c, where);

		if(sw < 0) {
			sw = sw + 4 * X + 6;
		} else {
			sw = sw + 4 * (X - Y) + 10;
		}

		Y = Y - 1;
		X = X + 1;

	} while(X <= Y);
}

void VGA::Rect(int left, int top, int right, int bottom, byte color, byte *where) {
	HLine(left, right, top, color, where);
    HLine(left, right, bottom, color, where);
    VLine(left, top, bottom, color, where);
    VLine(right, top, bottom, color, where);
}


void VGA::Line(int a, int b, int c, int d, int col, BYTE *where) {

	long u,s,v,d1x,d1y,d2x,d2y,m,n;
	int  i;

	u   = c-a;
	v   = d-b;
	d1x = sgn(u);
	d1y = sgn(v);
	d2x = sgn(u);
	d2y = 0;
	m   = abs(u);
	n   = abs(v);

	if (m<=n) {
		d2x = 0;
		d2y = sgn(v);
		m   = abs(v);
		n   = abs(u);
	}

	s = (int)(m / 2);

	for (i=0;i<round(m);i++) {
		PutPixel(a,b,col, where);
		s += n;
		if (s >= m) {
			s -= m;
			a += d1x;
			b += d1y;
		} else {
			a += d2x;
			b += d2y;
		}
	}
}

void VGA::HLine(int x1, int x2, int y, int col, BYTE *where) {
	int dif = abs(x2-x1);
	memset(where+((y<<8)+(y<<6)+x1), col, dif);
}


void VGA::VLine(int x, int y1, int y2, int col, BYTE *where) {
	int dif = abs(y2-y1);
	for(int i=0;i<dif;i++) {
		where[(y1<<8)+(y1<<6)+(i<<8)+(i<<6)+x] = col;
	}
}

void VGA::Flip(int VSync) {
	if(VSync) WaitRetrace();
	memcpy(VideoMem, BackBuffer, 64000);
}

int VGA::sgn(int a) {
	if (a > 0)  return +1;
	if (a < 0)  return -1;
	return 0;
}

int VGA::round(long a) {
	if ((a - (int)a)<0.5) {
		return floor(a);
	} else {
		return ceil(a);
	}
}
