#include <stdlib.h>
#include "xms.h"

static int xms_init(void);

static int init_done;
static unsigned long xms_ctl;


int xms_alloc_block(size_t bytes)
{
	unsigned char err;
	unsigned short kbsz, res, handle;

	if(!init_done) {
		if(xms_init() == -1) {
			return -1;
		}
		init_done = 1;
	}

	kbsz = (unsigned short)((bytes + 1023) / 1024);
	asm {
		mov ah, 9
		mov dx, bytes
		call [xms_ctl]
		mov res, ax
		mov handle, dx
		mov err, bl
	}

	if(!res) {
		switch(err) {
		case 0xa0:
			fprintf(stderr, "xms_alloc_block: out of XMS memory\n");
			break;

		case 0xa1:
			fprintf(stderr, "xms_alloc_block: all XMS memory block handles in use\n");
			break;

		default:
			fprintf(stderr, "xms_alloc_block: unknown error\n");
		}
		return -1;
	}
	return handle;
}

void xms_free_block(int handle)
{
	unsigned char err;
	unsigned short res, h16 = handle;
	asm {
		mov ah, 10
		mov dx, h16
		call [xms_ctl]
		mov res, ax
		mov err, bl
	}

	if(!res) {
		switch(err) {
		case 0xa2:
			fprintf(stderr, "xms_free_block: invalid handle: %d\n", handle);
			break;

		case 0xab:
			fprintf(stderr, "xms_free_block: handle %d is locked\n", handle);
			break;

		default:
			fprintf(stderr, "xms_free_block: unknown error\n");
		}
	}
}

int xms_move_block(void *cptr, int xhandle, int xoffs, size_t bytes, int dir)
{
}

static int xms_init(void)
{
	unsigned char present;
	unsigned short ver, xms_ctl_seg, xms_ctl_offs, res, sz;

	/* check if there is an XMS driver */
	asm {
		mov ax, 0x4300
		int 0x2f
		mov present, al
	}

	if(!present) {
		fprintf(stderr, "XMS driver not found.\n");
		return -1;
	}

	/* get the control function and driver version */
	asm {
		mov ax, 0x4310
		int 0x2f
		mov xms_ctl_seg, es
		mov xms_ctl_offs, bx

		mov word ptr [xms_ctl], bx
		mov word ptr [xms_ctl + 2], es

		xor ax, ax
		call [xms_ctl]
		mov ver, ax
	}
	printf("XMS (%x) control function at: %x:%x\n", ver, xms_ctl_seg, xms_ctl_offs);

	/* find out largest possible UMB */
	asm {
		mov ah, 0x10
		mov dx, 0xffff
		call [xms_ctl]
		mov res, ax
		mov sz, dx
	}
	printf("largest UMB block: %d paragraphs\n", sz);

	return 0;
}
